'use strict';

// ** Dependencies
const _ = require('lodash');
const util = require('util');
const callsite = require('callsite');

// ** Constants
const STRIP_COMMENTS = /(\/\/.*$)|(\/\*[\s\S]*?\*\/)|(\s*=[^,\)]*(('(?:\\'|[^'\r\n])*')|("(?:\\"|[^"\r\n])*"))|(\s*=[^,\)]*))/mg;
const ARGUMENT_NAMES = /([^\s,]+)/g;

/**
 * Parse the function body to determine what the parameter names are
 * @param func
 * @returns {Array|{index: number, input: string}}
 */
function getParameterNames(func) {
    var fnStr = func.toString();

    if (fnStr.match(/^[^ ()]+[ ]+=>/)) {
        fnStr = `(${fnStr.replace(/ =>/, ') =>')}`;
    }
    fnStr = fnStr.replace(STRIP_COMMENTS, '');

    var result = fnStr.slice(fnStr.indexOf('(') + 1, fnStr.indexOf(')')).match(ARGUMENT_NAMES);
    if (result === null)
        result = [];

    return result;
}

/**
 * Get Information about who called this function
 */
function getCallerInfo() {
    const site = callsite()[2];

    return {
        function_name: site.getFunctionName() || '<anonymous>',
        filename: site.getFileName(),
        line_number: site.getLineNumber()
    };
}

/**
 * Determine if a function is a class definition requires 'new' to be called.
 * @param v
 * @returns {boolean}
 */
function isClass(v) {
    return typeof v === 'function' && /^\s*class\s+/.test(v.toString());
}

function mapNamedArguments(func) {

    const context = this;
    const parameterNames = getParameterNames(func);

    return function(args) {
        const argArray = _.map(parameterNames, name => args[name]);
        return func.apply(context, argArray);
    }
}

/**
 * Return arguments to a proper arry
 * @param args
 * @constructor
 */
function Arguments(args) {
    return Array.prototype.slice.call(args);
}

// ** Exports
module.exports.getParameterNames = getParameterNames;
module.exports.getCallerInfo = getCallerInfo;
module.exports.isClass = isClass;
module.exports.mapNamedArguments = mapNamedArguments;
module.exports.Arguments = Arguments;