'use strict';

// ** Dependencies
const Promise = require('bluebird');
const Action = require('./actions');
const EventEmitter = require('./events').EventEmitter;
const functions = require('./functions');
const logger = require('./logging').createLogger('processes');

class Process extends EventEmitter {
    constructor(name, options) {
        super();

        this._name = name;
        this._options = options;
        this._steps = [];
    }

    get name() {
        return this._name;
    }

    get run() {
        // Create a new action that run each process
        const self = this;
        const action = Action(self.name, function () {

            const steps = self._steps;

            // Empty process, nothing to do
            if (steps.length === 0)
                return Promise.resolve();

            // Run the first action with the arguments supplied to the process
            let result = steps[0].action.run.apply(action, arguments);

            // Apply the result of the promise chain
            steps.slice(1).forEach(step => {

                const type = step.type;
                const action = step.action;

                const next = result[type];

                // Call the next action in the promise chain
                result = next.bind(result)(result => action.run(result));
            });

            // Run the first step
            return result;
        });

        // Forward Process Events as Action Events
        action.onAny((event, data) => this.emit(event, data));

        return action.run;
    }

    addStep(type, action, options) {

        // ** Emit events as process events
        action.onAny((event, data) => this.emit(`action:${action.name}:${event}`, data));

        this._steps.push({
            type: type,
            action: action
        });

        return action;
    }

    then(name, handler, options) {
        const action = Action(name, handler, options);

        this.addStep('then', action);

        return this;
    }

    tap(name, handler, options) {
        const action = Action(name, handler, options);

        this.addStep('tap', action);

        return this;
    }
}

function createProcess(name, options) {

    const process = new Process();
    return process;
}

// ** Exports
module.exports = createProcess;